package de.unikoblenz.fgbks.core.dmn.utils;

import de.unikoblenz.fgbks.core.dmn.domain.vdmn.VDmnDefinition;
import de.unikoblenz.fgbks.core.dmn.domain.vdmn.parser.SimpleVDmnParser;
import java.nio.charset.Charset;
import java.util.List;
import javax.enterprise.context.ApplicationScoped;
import javax.inject.Inject;
import org.apache.commons.io.IOUtils;
import org.camunda.bpm.dmn.engine.DmnDecision;
import org.camunda.bpm.dmn.engine.DmnEngineConfiguration;
import org.camunda.bpm.model.dmn.DmnModelInstance;
import org.camunda.bpm.model.dmn.impl.DmnParser;

/**
 * Application scoped service, which creates a {@link DmnObjectContainer} with the method {@link
 * DmnService#getDmnServiceFromXml(String)}. This container contained the different model types of a
 * dmn table.
 */
@ApplicationScoped
public class DmnService {

  @Inject SimpleVDmnParser simpleDmnParser;

  /**
   * Get the {@link DmnObjectContainer} with the given xml string of a dmn table.
   *
   * @param dmnXmlString the xml of a dmn table
   * @return a new {@link DmnObjectContainer}
   */
  public DmnObjectContainer getDmnServiceFromXml(String dmnXmlString) {
    List<DmnDecision> dmnDecisions =
        DmnEngineConfiguration.createDefaultDmnEngineConfiguration()
            .buildEngine()
            .parseDecisions(IOUtils.toInputStream(dmnXmlString, Charset.defaultCharset()));
    DmnModelInstance dmnModel =
        new DmnParser()
            .parseModelFromStream(IOUtils.toInputStream(dmnXmlString, Charset.defaultCharset()));
    VDmnDefinition vDmnDefinition = simpleDmnParser.getVDmnDefinition(dmnModel);
    return new DmnObjectContainer(dmnDecisions, dmnModel, vDmnDefinition);
  }
}
