package de.unikoblenz.fgbks.core.dmn.verification.result.actions;

import static de.unikoblenz.fgbks.core.dmn.verification.result.actions.Action.ACTION_SHOW_INPUT_COLUMNS;
import static de.unikoblenz.fgbks.core.dmn.verification.result.actions.Action.ACTION_SHOW_INPUT_ENTRIES;
import static de.unikoblenz.fgbks.core.dmn.verification.result.actions.Action.ACTION_SHOW_OUTPUT_COLUMNS;
import static de.unikoblenz.fgbks.core.dmn.verification.result.actions.Action.ACTION_SHOW_OUTPUT_ENTRIES;
import static de.unikoblenz.fgbks.core.dmn.verification.result.actions.Action.ACTION_SHOW_RULES;

import de.unikoblenz.fgbks.base.builder.DefaultBuilder;
import de.unikoblenz.fgbks.base.domain.Name;
import java.util.ArrayList;
import java.util.List;
import org.apache.commons.lang3.Validate;

public class VerificationFix {

  private static final Name DEFAULT_FIX_NAME = new Name("Fix");
  private static final Name DEFAULT_SHOW_NAME = new Name("Show");

  public static VerificationFix SHOW_RULES;
  public static VerificationFix SHOW_INPUT_ENTRIES;
  public static VerificationFix SHOW_OUTPUT_ENTRIES;
  public static VerificationFix SHOW_INPUT_COLUMNS;
  public static VerificationFix SHOW_OUTPUT_COLUMNS;

  static {
    SHOW_RULES =
        VerificationFix.getBuilder()
            .withFixName(DEFAULT_SHOW_NAME)
            .addAction(ACTION_SHOW_RULES)
            .build();
    SHOW_INPUT_ENTRIES =
        VerificationFix.getBuilder()
            .withFixName(DEFAULT_SHOW_NAME)
            .addAction(ACTION_SHOW_INPUT_ENTRIES)
            .build();
    SHOW_OUTPUT_ENTRIES =
        VerificationFix.getBuilder()
            .withFixName(DEFAULT_SHOW_NAME)
            .addAction(ACTION_SHOW_OUTPUT_ENTRIES)
            .build();
    SHOW_INPUT_COLUMNS =
        VerificationFix.getBuilder()
            .withFixName(DEFAULT_SHOW_NAME)
            .addAction(ACTION_SHOW_INPUT_COLUMNS)
            .build();
    SHOW_OUTPUT_COLUMNS =
        VerificationFix.getBuilder()
            .withFixName(DEFAULT_SHOW_NAME)
            .addAction(ACTION_SHOW_OUTPUT_COLUMNS)
            .build();
  }

  private Name fixName;
  private List<Action> actions;

  private VerificationFix() {
    fixName = DEFAULT_FIX_NAME;
    actions = new ArrayList<>();
  }

  private static Builder getBuilder() {
    return new VerificationFix().new Builder();
  }

  private class Builder extends DefaultBuilder<VerificationFix> {

    public Builder withFixName(Name fixName) {
      value.fixName = fixName;
      return this;
    }

    public Builder addAction(Action action) {
      value.actions.add(Validate.notNull(action));
      return this;
    }

    @Override
    protected void validate() {
      super.validate();
      Validate.notNull(value.fixName);
    }
  }
}
